/*
FILE: main.c
DESCRIPTION: Test file demonstrating the usage of the memcard library
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 22, 2002
*/


//Includes---------------------------------------------------------------------
#include <stdio.h>
#include "memcardlib.h"


//Functions--------------------------------------------------------------------
void print_stats(void)
{
	int i;
	int block_size, total_blocks, free_blocks;

	for (i = 0; i < memcardlib_num_supported_memcards(); i++)
	{		
		printf("Info for memord card %d\n", i);

		switch (memcardlib_get_card_type(i))
		{
			case PS2MC_TYPE_EMPTY: puts("\tNo memory card"); break;
			case PS2MC_TYPE_PS1: puts("\tPS1 memory card"); break;
			case PS2MC_TYPE_PS2: puts("\tPS2 memory card"); break;
			case PS2MC_TYPE_POCKETSTATION: puts("\tPocketstation"); break;
		}
			
		memcardlib_get_card_block_info(i, &block_size, &total_blocks, &free_blocks);
		printf("\tBlock size: %d\n", block_size);
		printf("\tTotal blocks: %d\n", total_blocks);
		printf("\tFree blocks: %d\n", free_blocks);
		printf("\tTotal free memory: %d\n", block_size * free_blocks);
		printf("\tTotal memory: %d\n", block_size * total_blocks);

		switch (memcardlib_is_card_formatted(i))
		{
			case 0: puts("\tMemory card is unformatted"); break;
			case 1: puts("\tMemory card is formatted"); break;
		}
		
		switch (memcardlib_get_card_status(i))
		{
			case PS2MC_STAT_NOTCON: puts("\tMemory card is not connected"); break;
			case PS2MC_STAT_READY: puts("\tMemory card is ready"); break;
			case PS2MC_STAT_BUSY: puts("\tMemory card is busy"); break;
			case PS2MC_STAT_ERROR: puts("\tMemory card error"); break;
		}
	}
}

void read_test(int card, const char* filename)
{
	void* mcfile;
	char buffer[256] = {0};

	mcfile = memcardlib_open(card, filename, 0);

	if (mcfile)
	{
		int num_read;
	
		printf("Opened %s on memory card %d for reading\n", filename, card);		

		num_read = memcardlib_read(mcfile, buffer, sizeof(buffer));
		printf("Number of bytes read: %d\n", num_read);
		printf("Contents of file: %s\n", buffer);

		memcardlib_close(mcfile);
	}
	else
		printf("Couldn't open memory card %d for reading", card);
}

void write_test(int card, const char* filename)
{
	void* mcfile;
	char* buffer = "Text from the the memcardlib sample";

	mcfile = memcardlib_open(card, filename, 1);

	if (mcfile)
	{
		int num_written;
	
		printf("Opened %s on memory card %d for writing\n", filename, card);		

		num_written = memcardlib_write(mcfile, buffer, strlen(buffer));
		printf("Number of bytes written: %d\n", num_written);
		printf("Contents of file: %s\n", buffer);

		memcardlib_close(mcfile);
	}
	else
		printf("Couldn't open memory card %d for writing", card);
}

int main(int argc, char** argv)
{
	int active_memcard = 0;	//Which memory card slot to use, 0 or 1
	int run_mode = 0;	//0 for read, 1 for write, 2 for delete, 3 for stats
	const char* filename = "memcardlib_test.txt";	//Name of file to create on memory card

	//The user must pass in an argument for the program to run	
	if (argc < 2)
	{
		puts("Usage: main {read | write | delete | stats}");
		return 0;
	}

	//Parse arguments
	if (!strcmp(argv[1], "read"))
		run_mode = 0;
	else if (!strcmp(argv[1], "write"))
		run_mode = 1;
	else if (!strcmp(argv[1], "delete"))
		run_mode = 2;
	else if (!strcmp(argv[1], "stats"))
		run_mode = 3;
	else
	{
		puts("Usage: main {read | write | delete | stats}");
		return 0;
	}

	//Initialize memcard library
	memcardlib_initialize();

	//If there's no memory card in the selected slot, bail out
	if (memcardlib_get_card_type(active_memcard) == PS2MC_TYPE_EMPTY)
	{
		printf("A memory card must be in slot %d for this sample to work", active_memcard);
		memcardlib_shutdown();
		return 0;
	}		
	
	//Run the desired operation
	switch (run_mode)
	{
		case 0: 
		{
			if (memcardlib_file_exists(active_memcard, filename))
			{
				printf("%s exists on memory card %d\n", filename, active_memcard);
				read_test(active_memcard, filename); 
			}
			else
				printf("%s doesn't exist on memory card %d\n", filename, active_memcard);

			break;
		}
		case 1: write_test(active_memcard, filename); break;
		case 2: 
		{
			if (memcardlib_file_exists(active_memcard, filename))
			{
				memcardlib_delete_file(active_memcard, filename);
				printf("%s deleted from memory card %d\n", filename, active_memcard);
			}
			else
				printf("%s doesn't exist on memory card %d\n", filename, active_memcard);
			break;
		}
		case 3: print_stats(); break;
	}

	memcardlib_shutdown();
}
