/*
FILE: memcardlib.c
DESCRIPTION: The memcardlib functions
WRITTEN BY: Derek Nedelman (nedelman@netzero.net)
DATE: August 21, 2002
HOME PAGE: http://www.playstation2-linux.com/projects/memcardlib
*/

//Includes---------------------------------------------------------------------
#include <sys/stat.h>

#include <sys/ioctl.h>
#include <sys/mount.h>
#include <unistd.h>
#include <memory.h>
#include <string.h>
#include <stdio.h>

#include "memcardlib.h"


//Structures-------------------------------------------------------------------
typedef struct
{
	int card;			//Index of owner structure in global g_memcard array
	int is_open;			//Whether or not this "file" is being used
	char path[PS2MC_NAME_MAX + 1];	//Name of file being used
	int path_length;		//Length of string in 'path'
	int cursor;			//For reading or writing - offset into current file

	int read_or_write_mode;		//0 if reading, 1 if writing	
	FILE* write_file;		//If writing, pointer to file being written
}memcard_file_t;

typedef struct
{
	int fd;				//File descriptor
	struct ps2mc_cardinfo info;	//Current info about the memory card
	memcard_file_t file;		//"File" for this memory card. Only one file per memory card, obviously
}memcard_t;


//Globals----------------------------------------------------------------------
#define MAX_MEMCARDS 2
static memcard_t g_memcards[MAX_MEMCARDS];

static const char* g_memcard_names[MAX_MEMCARDS] = 
{
	"mc00",
	"mc10"
};

static const char* g_device_names[MAX_MEMCARDS] =
{
	"/dev/ps2mc00",
	"/dev/ps2mc10"
};


//Prototypes for local functions-----------------------------------------------
void memcardlib_close_write_file(memcard_file_t* memcard_file);
void memcardlib_update(int card);


//Functions--------------------------------------------------------------------
void memcardlib_initialize(void)
{
	int i;	
		
	memset(g_memcards, 0, sizeof(g_memcards));
	for (i = 0; i < MAX_MEMCARDS; i++)
	{
		g_memcards[i].fd = open(g_device_names[i], O_RDWR);
		memset(&g_memcards[i].file, 0, sizeof(g_memcards[i].file));
		g_memcards[i].file.card = i;

		//Make a local directory for mounting the card
		mkdir(g_memcard_names[i], 0);
	}
}

void memcardlib_shutdown(void)
{
	int i;
	
	for (i = 0; i < MAX_MEMCARDS; i++)
	{		
		if (g_memcards[i].fd >= 0)
		{
			close(g_memcards[i].fd);
			g_memcards[i].fd = -1;
		}

		//If the write_file happens to be open, close it
		if (g_memcards[i].file.write_file)
			memcardlib_close_write_file(&g_memcards[i].file);
		
		//Remove the directory it was using
		rmdir(g_memcard_names[i]);
	}
}

int memcardlib_num_supported_memcards(void)
{
	return MAX_MEMCARDS;
}

int memcardlib_get_card_type(int card)
{
	memcardlib_update(card);
	return g_memcards[card].info.type;
}

int memcardlib_get_card_status(int card)
{
	memcardlib_update(card);
	return g_memcards[card].info.busy;
}

void memcardlib_get_card_block_info(int card, int* block_size, int* total_blocks, int* free_blocks)
{
	memcardlib_update(card);

	if (block_size)
		*block_size = g_memcards[card].info.blocksize;
	if (total_blocks)
		*total_blocks = g_memcards[card].info.totalblocks;
	if (free_blocks)
		*free_blocks = g_memcards[card].info.freeblocks;
}

int memcardlib_is_card_formatted(int card)
{
	memcardlib_update(card);
	return g_memcards[card].info.formatted;
}

void* memcardlib_open(int card, const char* name, int read_or_write)
{
	//Only open the file if:
	//	1)The file isn't already open
	//	2)The slot it's in was opened
	//	3)A writable card type is available
	//	4)The card available is formatted
	//	5)The card is ready to be written to
	if (g_memcards[card].file.is_open ||
        g_memcards[card].fd < 0 ||
	    memcardlib_get_card_type(card) < PS2MC_TYPE_PS1 ||
        !memcardlib_is_card_formatted(card) ||
        memcardlib_get_card_status(card) != PS2MC_STAT_READY)
	{
		return 0;	
	}

	if (read_or_write == 0)
	{
		//Reading
		sprintf(g_memcards[card].file.path, "/%s", name);
	}
	else
	{
		//Writing

		int result;
		
		//Mount the card				
		result = mount(g_device_names[card], g_memcard_names[card], "ps2mcfs", 0, 0);

		//If result is nonzero there was an error
		if (result)
			return 0;
		
		//Open the file on the card
		sprintf(g_memcards[card].file.path, "%s/%s", g_memcard_names[card], name);
		g_memcards[card].file.write_file = fopen(g_memcards[card].file.path, "wb");
	}

	g_memcards[card].file.path_length = strlen(g_memcards[card].file.path);
	g_memcards[card].file.cursor = 0;
	g_memcards[card].file.is_open = 1;

	return &g_memcards[card].file;
}

void memcardlib_close(void* file)
{
	memcard_file_t* memcard_file = (memcard_file_t*)file;

	if (!memcard_file->is_open)
		return;

	if (memcard_file->write_file)
	{
		memcardlib_close_write_file(memcard_file);
	}

	memcard_file->path[0] = 0;
	memcard_file->path_length = 0;
	memcard_file->cursor = 0;	
	memcard_file->is_open = 0;	
}

int memcardlib_read(void* file, void* data, int num_bytes)
{
	memcard_file_t* memcard_file = (memcard_file_t*)file;
	struct ps2mc_arg arg;

	if (!memcard_file->is_open)
		return 0;

	arg.path = memcard_file->path;
	arg.pathlen = memcard_file->path_length;
	arg.mode = 0;
	arg.pos = memcard_file->cursor;
	arg.data = (unsigned char*)data;
	arg.count = num_bytes;
	memset(arg.reserved, 0, sizeof(arg.reserved));

	num_bytes = ioctl(g_memcards[memcard_file->card].fd, PS2MC_IOCREAD, &arg);

	memcard_file->cursor += num_bytes;

	return num_bytes;
}

int memcardlib_write(void* file, const void* data, int num_bytes)
{
	memcard_file_t* memcard_file = (memcard_file_t*)file;

	if (!memcard_file->is_open)
		return 0;

	num_bytes = fwrite(data, 1, num_bytes, memcard_file->write_file);

	memcard_file->cursor += num_bytes;

	return num_bytes;
}

int memcardlib_get_pos(void* file)
{
	memcard_file_t* memcard_file = (memcard_file_t*)file;
	return memcard_file->cursor;
}

int memcardlib_file_exists(int card, const char* name)
{
	struct ps2mc_arg arg;
	char path[PS2MC_NAME_MAX + 1];
	unsigned char read_buffer[1];
	int num_bytes;

	sprintf(path, "/%s", name);

	arg.path = path;
	arg.pathlen = strlen(path);
	arg.mode = 0;
	arg.pos = 0;
	arg.data = read_buffer;
	arg.count = 1;
	memset(arg.reserved, 0, sizeof(arg.reserved));

	num_bytes = ioctl(g_memcards[card].fd, PS2MC_IOCREAD, &arg);

	return num_bytes == 1;
}

void memcardlib_delete_file(int card, const char* name)
{
	if (g_memcards[card].file.is_open)
	{
		//The card is being written or read from at the moment
	}
	else
	{
		char filename[256];		
		int result;
		
		//Mount the card				
		result = mount(g_device_names[card], g_memcard_names[card], "ps2mcfs", 0, 0);

		//If result is nonzero there was an error
		if (result)
			return;
		
		sprintf(filename, "%s/%s", g_memcard_names[card], name);
		unlink(filename);

		//Unmount the memcard
		umount(g_memcard_names[card]);
	}
}


void memcardlib_update(int card)
{
	if (g_memcards[card].fd >= 0)
	{
		ioctl(g_memcards[card].fd, PS2MC_IOCGETINFO, &g_memcards[card].info);
	}
}

void memcardlib_close_write_file(memcard_file_t* memcard_file)
{
	//Closes memcard_file->write_file

	//Close the file on the memory card
	fclose(memcard_file->write_file);
	memcard_file->write_file = 0;

	//Unmount the memcard
	umount(g_memcard_names[memcard_file->card]);
}

